@@ -10,6 +10,7 @@ import os
 import re
 import warnings
 import weakref
+from collections import defaultdict
 from datetime import datetime, timedelta
 from glob import has_magic
 from typing import Optional, Tuple
@@ -24,7 +25,7 @@ from azure.storage.blob._models import BlobBlock, BlobProperties, BlobType
 from azure.storage.blob._shared.base_client import create_configuration
 from azure.storage.blob.aio import BlobServiceClient as AIOBlobServiceClient
 from azure.storage.blob.aio._list_blobs_helper import BlobPrefix
-from fsspec.asyn import AsyncFileSystem, get_loop, get_running_loop, sync, sync_wrapper
+from fsspec.asyn import AsyncFileSystem, get_loop, sync, sync_wrapper
 from fsspec.spec import AbstractBufferedFile
 from fsspec.utils import infer_storage_options
 
@@ -82,6 +83,18 @@ def make_callback(key, callback):
     return wrapper
 
 
+def get_running_loop():
+    # this was removed from fsspec in https://github.com/fsspec/filesystem_spec/pull/1134
+    if hasattr(asyncio, "get_running_loop"):
+        return asyncio.get_running_loop()
+    else:
+        loop = asyncio._get_running_loop()
+        if loop is None:
+            raise RuntimeError("no running event loop")
+        else:
+            return loop
+
+
 def _coalesce_version_id(*args) -> Optional[str]:
     """Helper to coalesce a list of version_ids down to one"""
     version_ids = set(args)
@@ -499,6 +512,12 @@ class AzureBlobFileSystem(AsyncFileSystem):
             )
         return super().info(path)
 
+    def modified(self, path: str) -> datetime:
+        return self.info(path)["last_modified"]
+
+    def created(self, path: str) -> datetime:
+        return self.info(path)["creation_time"]
+
     async def _info(self, path, refresh=False, **kwargs):
         """Give details of entry at path
         Returns a single dictionary, with exactly the same information as ``ls``
@@ -650,12 +669,20 @@ class AzureBlobFileSystem(AsyncFileSystem):
         delimiter: str = "/",
         return_glob: bool = False,
         version_id: Optional[str] = None,
+        versions: bool = False,
+        **kwargs,
     ):
+        if (version_id or versions) and not self.version_aware:
+            raise ValueError(
+                "version_id/versions cannot be specified if the filesystem "
+                "is not version aware"
+            )
+
         if (
             target_path not in self.dircache
             or return_glob
-            or self.version_aware
-            and not any(
+            or versions
+            or not any(
                 match_blob_version(b, version_id) for b in self.dircache[target_path]
             )
         ):
@@ -666,12 +693,8 @@ class AzureBlobFileSystem(AsyncFileSystem):
                 ) as cc:
                     path = path.strip("/")
                     include = ["metadata"]
-                    if version_id is not None:
-                        if not self.version_aware:
-                            raise ValueError(
-                                "version_id cannot be specified if the "
-                                "filesystem is not version aware"
-                            )
+                    if version_id is not None or versions:
+                        assert self.version_aware
                         include.append("versions")
                     blobs = cc.walk_blobs(include=include, name_starts_with=path)
 
@@ -718,11 +741,15 @@ class AzureBlobFileSystem(AsyncFileSystem):
                     target_path=target_path,
                     return_glob=return_glob,
                     version_id=version_id,
+                    versions=versions,
                 )
                 if return_glob:
                     return finalblobs
                 finalblobs = await self._details(
-                    outblobs, target_path=target_path, version_id=version_id
+                    outblobs,
+                    target_path=target_path,
+                    version_id=version_id,
+                    versions=versions,
                 )
                 if not finalblobs:
                     if not await self._exists(target_path):
@@ -742,6 +769,7 @@ class AzureBlobFileSystem(AsyncFileSystem):
         delimiter: str = "/",
         return_glob: bool = False,
         version_id: Optional[str] = None,
+        versions: bool = False,
         **kwargs,
     ):
         """
@@ -764,6 +792,9 @@ class AzureBlobFileSystem(AsyncFileSystem):
         version_id: str
             Specific blob version to list
 
+        versions: bool
+            If True, list all versions
+
         return_glob: bool
 
         """
@@ -785,8 +816,15 @@ class AzureBlobFileSystem(AsyncFileSystem):
                 delimiter=delimiter,
                 return_glob=return_glob,
                 version_id=version_id,
+                versions=versions,
             )
 
+        if versions:
+            for entry in output:
+                entry_version_id = entry.get("version_id")
+                if entry_version_id:
+                    entry["name"] += f"?versionid={entry_version_id}"
+
         if detail:
             return output
 
@@ -799,6 +837,7 @@ class AzureBlobFileSystem(AsyncFileSystem):
         return_glob: bool = False,
         target_path="",
         version_id: Optional[str] = None,
+        versions: bool = False,
         **kwargs,
     ):
         """
@@ -816,6 +855,9 @@ class AzureBlobFileSystem(AsyncFileSystem):
         version_id: str
             Specific target version to be returned
 
+        versions: bool
+            If True, return all versions
+
         Returns
         -------
         List of dicts
@@ -883,7 +925,11 @@ class AzureBlobFileSystem(AsyncFileSystem):
                 # This handles the case where path is a file passed to ls
                 return output
             output = await filter_blobs(
-                output, target_path, delimiter, version_id=version_id
+                output,
+                target_path,
+                delimiter,
+                version_id=version_id,
+                versions=versions,
             )
 
         return output
@@ -1154,57 +1200,56 @@ class AzureBlobFileSystem(AsyncFileSystem):
             else:
                 raise
 
-    async def _rm(self, path, recursive=False, maxdepth=None, **kwargs):
+    async def _rm(
+        self,
+        path,
+        recursive=False,
+        maxdepth=None,
+        delimiter="/",
+        expand_path=True,
+        **kwargs,
+    ):
         """Delete files.
+
         Parameters
         ----------
         path: str or list of str
             File(s) to delete.
         recursive: bool
+            Defaults to False.
             If file(s) are directories, recursively delete contents and then
-            also remove the directory
+            also remove the directory.
+            Only used if `expand_path`.
+
         maxdepth: int or None
+            Defaults to None.
             Depth to pass to walk for finding files to delete, if recursive.
             If None, there will be no limit and infinite recursion may be
             possible.
+            Only used if `expand_path`.
+        expand_path: bool
+            Defaults to True.
+            If False, `self._expand_path` call will be skipped. This is more
+            efficient when you don't need the operation.
         """
-        path = await self._expand_path(
-            path, recursive=recursive, maxdepth=maxdepth, with_parent=True
-        )
-        for p in reversed(path):
-            await self._rm_file(p)
-        self.invalidate_cache()
+        if expand_path:
+            path = await self._expand_path(
+                path, recursive=recursive, maxdepth=maxdepth, with_parent=True
+            )
+        elif isinstance(path, str):
+            path = [path]
 
-    rm = sync_wrapper(_rm)
-
-    async def _rm_file(self, path, delimiter="/", **kwargs):
-        """
-        Delete a given file
-
-        Parameters
-        ----------
-        path: str
-            Path to file to delete
-
-        delimiter: str
-            Delimiter to use when splitting the path
-        """
+        grouped_files = defaultdict(list)
         try:
-            if await self._isfile(path.rstrip("/")):
-                kind = "file"
-            else:
-                kind = "directory"
+            for p in reversed(path):
+                container_name, p, _ = self.split_path(p, delimiter=delimiter)
+                if p != "":
+                    grouped_files[container_name].append(p.rstrip(delimiter))
+                else:
+                    await self._rmdir(container_name)
 
-            container_name, path, _ = self.split_path(path, delimiter=delimiter)
-            if path != "":
-                async with self.service_client.get_container_client(
-                    container=container_name
-                ) as cc:
-                    await cc.delete_blob(path.rstrip(delimiter))
-            elif kind == "directory":
-                await self._rmdir(container_name)
-            else:
-                raise RuntimeError(f"Unable to remove {path}")
+            for container_name, files in grouped_files.items():
+                await self._rm_files(container_name, files)
         except ResourceNotFoundError:
             pass
         except FileNotFoundError:
@@ -1212,9 +1257,32 @@ class AzureBlobFileSystem(AsyncFileSystem):
         except Exception as e:
             raise RuntimeError("Failed to remove %s for %s", path, e)
 
-        self.invalidate_cache(self._parent(path))
+        self.invalidate_cache()
 
-    sync_wrapper(_rm_file)
+    rm = sync_wrapper(_rm)
+
+    async def _rm_files(self, container_name, files, **kwargs):
+        """
+        Delete the given file(s)
+
+        Parameters
+        ----------
+        path: str or list of str
+            File(s) to delete.
+        """
+        async with self.service_client.get_container_client(
+            container=container_name
+        ) as cc:
+            exs = await asyncio.gather(
+                *([cc.delete_blob(file) for file in files]), return_exceptions=True
+            )
+            for ex in exs:
+                if ex is not None:
+                    raise ex
+        for file in files:
+            self.invalidate_cache(self._parent(file))
+
+    sync_wrapper(_rm_files)
 
     def rmdir(self, path: str, delimiter="/", **kwargs):
         sync(self.loop, self._rmdir, path, delimiter=delimiter, **kwargs)
@@ -1606,13 +1674,13 @@ class AzureBlobFileSystem(AsyncFileSystem):
             async with self.service_client.get_blob_client(
                 container_name, path.rstrip(delimiter)
             ) as bc:
+                stream = await bc.download_blob(
+                    raw_response_hook=make_callback(
+                        "download_stream_current", callback
+                    ),
+                    version_id=version_id,
+                )
                 with open(lpath, "wb") as my_blob:
-                    stream = await bc.download_blob(
-                        raw_response_hook=make_callback(
-                            "download_stream_current", callback
-                        ),
-                        version_id=version_id,
-                    )
                     await stream.readinto(my_blob)
         except ResourceNotFoundError as exception:
             raise FileNotFoundError from exception
